#include <stdlib.h>
#include <kernel/mm.h>
void
*malloc (size_t size)
{
    uintptr_t  start = os.mem_ph.available_s;
    mm_head_t *head  = (void *) start;
       size_t  actual_size;
    uintptr_t  current;
    uintptr_t  next;
    uintptr_t  new;
    uintptr_t  up_to;
          int  counter;
    //
    // Arrotonda in eccesso il valore di «size», in modo che sia un
    // multiplo della dimensione di «mm_head_t». Altrimenti, la
    // collocazione dei blocchi successivi può avvenire in modo
    // non allineato.
    //
    size = (size + (sizeof (mm_head_t)) -1);
    size = size / (sizeof (mm_head_t));
    size = size * (sizeof (mm_head_t));
    //
    // Cerca un blocco libero di dimensione sufficiente.
    //
    counter = 2;
    while (counter)
      {
        //
        // Annota la posizione attuale e quella successiva.
        //
        current = (uintptr_t) head;
           next = head->next * (sizeof (mm_head_t));
        //
        if (next == start)
          {
            up_to = os.mem_ph.available_e;
          }
        else
          {
            up_to = next;
          }
        //
        // Se è stato raggiunto il primo elemento, decrementa il
        // contatore di una unità. Se è già a zero, esce.
        //
        if (current == start)
          {
            counter--;
            if (counter == 0) break;
          }
        //
        // Controlla se si tratta di un blocco libero.
        //

        if (! head->allocated)
          {
            //
            // Il blocco è libero: si deve determinarne la dimensione.
            //
            if (current == start && next == start)
              {
                //
                // Si tratta del primo e unico elemento della lista.
                //
                actual_size = os.mem_ph.available_e - (sizeof (mm_head_t));
              }
            else
              {
                actual_size = up_to - current - (sizeof (mm_head_t));
              }
            //
            // Si verifica che sia capiente.
            //
            if (actual_size >= size + ((sizeof (mm_head_t)) * 2))
              {
                //
                // C'è spazio per dividere il blocco.
                //
                new = current + size + (sizeof (mm_head_t));
                //
                // Aggiorna l'intestazione attuale.
                //
                head->allocated = 1;
                head->next = new / (sizeof (mm_head_t));
                //
                // Predispone l'intestazione successiva.
                //
                head = (void *) new;
                head->allocated = 0;
                head->next = next / (sizeof (mm_head_t));
                //
                // Restituisce l'indirizzo iniziale dello spazio libero,
                // successivo all'intestazione.
                //
                return (void *) (current + (sizeof (mm_head_t)));
              }
            else if (actual_size >= size)
              {
                //
                // Il blocco va usato per intero.
                //
                head->allocated = 1;
                //
                // Restituisce l'indirizzo iniziale dello spazio libero,
                // successivo all'intestazione.
                //
                return (void *) (current + (sizeof (mm_head_t)));
              }
          }
        //
        // Il blocco è allocato, oppure è di dimensione insufficiente;
        // pertanto occorre passare alla posizione successiva.
        //
        head = (void *) next;
      }       
    //
    // Essendo terminato il ciclo precedente, vuol dire
    // che non ci sono spazi disponibili.
    //
    return NULL;
}
